-- Adresse (_aid_, plz, ort, strasse, nummer)
CREATE SEQUENCE adresse_sequence;
CREATE TABLE Adresse (
  aid      NUMERIC(8)    PRIMARY KEY DEFAULT nextval('adresse_sequence'),
  plz      VARCHAR(10)   NOT NULL,
  ort      VARCHAR(50)   NOT NULL,
  strasse  VARCHAR(200)  NOT NULL,
  nummer   VARCHAR(10)   NOT NULL,
  UNIQUE(plz, ort, strasse, nummer)
);
ALTER SEQUENCE adresse_sequence OWNED BY Adresse.aid;

-- Standort (_sid_, bezeichnung, adresse:Adresse.aid, leiter:Angestellter.svnr)
CREATE SEQUENCE standort_sequence;
CREATE TABLE Standort (
  sid          NUMERIC(8)   PRIMARY KEY DEFAULT nextval('standort_sequence'),
  bezeichnung  VARCHAR(50)  NOT NULL,
  adresse      NUMERIC(8)   NOT NULL REFERENCES Adresse(aid),
  leiter       NUMERIC(10)  NOT NULL
);
ALTER SEQUENCE standort_sequence OWNED BY Standort.sid;

-- Angestellter (_svnr_, vname, nname, gebdatum, gehalt, adresse:Adresse.aid, standort:Standort.sid)
CREATE TABLE Angestellter (
  svnr      NUMERIC(10)   PRIMARY KEY,
  vname     VARCHAR(50)   NOT NULL,
  nname     VARCHAR(50)   NOT NULL,
  gebdatum  DATE          NOT NULL,
  gehalt    NUMERIC(7,2)  NOT NULL,
  adresse   NUMERIC(8)    NOT NULL REFERENCES Adresse(aid),
  standort  NUMERIC(8)    NOT NULL REFERENCES Standort(sid) DEFERRABLE INITIALLY DEFERRED
);

ALTER TABLE Standort ADD CONSTRAINT fk_leiter
  FOREIGN KEY (leiter) REFERENCES Angestellter(svnr)
  DEFERRABLE INITIALLY DEFERRED;

-- Manager (_svnr_:Angestellter.svnr, bonus)
CREATE TABLE Manager (
  svnr   NUMERIC(10)   PRIMARY KEY REFERENCES Angestellter(svnr),
  bonus  NUMERIC(7,2)  NOT NULL
);

-- Artikel (_ean_, bezeichnung, typ)
CREATE TABLE Artikel (
  ean          NUMERIC(13)  PRIMARY KEY,
  bezeichnung  VARCHAR(50)  NOT NULL,
  typ          VARCHAR(20)  NOT NULL
);

-- Serie (_vorgaenger_:Artikel.ean, _nachfolger_:Artikel.ean)
CREATE TABLE Serie (
  vorgaenger  NUMERIC(13)  NOT NULL REFERENCES Artikel(ean),
  nachfolger  NUMERIC(13)  NOT NULL REFERENCES Artikel(ean),
  CHECK (vorgaenger <> nachfolger),
  UNIQUE (vorgaenger),
  PRIMARY KEY (vorgaenger, nachfolger)
);

-- Land (_code_, name, kontinent)
CREATE TYPE kontinent_t AS ENUM (
  'Nordamerika', 'Suedamerika', 'Europa',
  'Afrika', 'Asien', 'Australien'
);
CREATE TABLE Land (
  code       VARCHAR(2)   PRIMARY KEY,
  name       VARCHAR(50)  NOT NULL,
  kontinent  kontinent_t  NOT NULL
);

-- Preis (_datum_, _land_:Land.code, _artikel_:Artikel.ean, preis, steuer)
CREATE TABLE Preis (
  datum    DATE          NOT NULL,
  land     VARCHAR(2)    NOT NULL REFERENCES Land(code),
  artikel  NUMERIC(13)   NOT NULL REFERENCES Artikel(ean),
  preis    NUMERIC(7,2)  NOT NULL,
  steuer   NUMERIC(3)    NOT NULL,
  CHECK (steuer BETWEEN 0 AND 100),
  PRIMARY KEY (datum, land, artikel)
);

-- Kunde (_knr_, vname, nname, adresse:Adresse.aid)
CREATE SEQUENCE kunde_sequence
  START 1000001 INCREMENT BY 10
  MINVALUE 1000001 MAXVALUE 9999991;
CREATE TABLE Kunde (
  knr      NUMERIC(7)   PRIMARY KEY DEFAULT nextval('kunde_sequence'),
  vname    VARCHAR(50)  NOT NULL,
  nname    VARCHAR(50)  NOT NULL,
  adresse  NUMERIC(8)   NOT NULL REFERENCES Adresse(aid)
);
ALTER SEQUENCE kunde_sequence OWNED BY Kunde.knr;

-- Bestellung (_bid_, zeitpunkt, kunde:Kunde.knr)
CREATE SEQUENCE bestellung_sequence
  START 1000000001
  MINVALUE 1000000001 MAXVALUE 9999999999;
CREATE TABLE Bestellung (
  bid        NUMERIC(10)  PRIMARY KEY DEFAULT nextval('bestellung_sequence'),
  zeitpunkt  TIMESTAMP    NOT NULL DEFAULT CURRENT_TIMESTAMP,
  kunde      NUMERIC(7)   NOT NULL REFERENCES Kunde(knr)
);
ALTER SEQUENCE bestellung_sequence OWNED BY Bestellung.bid;

-- bezahlt (_bid_:Bestellung.bid, datum)
CREATE TABLE bezahlt (
  bid        NUMERIC(10)  PRIMARY KEY REFERENCES Bestellung(bid),
  datum      DATE         NOT NULL
);

-- beinhaltet (_bestellung_:Bestellung.bid, _pdatum_:Preis.datum, _pland_:Preis.land, _partikel_:Preis.artikel, anzahl)
CREATE TABLE beinhaltet (
  bestellung  NUMERIC(10)  NOT NULL REFERENCES Bestellung(bid),
  pdatum      DATE         NOT NULL,
  pland       VARCHAR(2)   NOT NULL,
  partikel    NUMERIC(13)  NOT NULL,
  anzahl      NUMERIC(5)   NOT NULL,
  PRIMARY KEY (bestellung, pdatum, pland, partikel),
  FOREIGN KEY (pdatum, pland, partikel) REFERENCES Preis
);

-- lagern (_artikel_:Artikel.ean, _standort_:Standort.sid, anzahl)
CREATE TABLE lagern (
  artikel     NUMERIC(13)  NOT NULL REFERENCES Artikel(ean),
  standort    NUMERIC(8)   NOT NULL REFERENCES Standort(sid),
  anzahl      NUMERIC(5)   NOT NULL,
  PRIMARY KEY (artikel, standort)
);
